package com.search.supersearch.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.search.Hit;
import com.search.supersearch.model.pojo.DocsIndex;
import com.search.supersearch.model.pojo.PlatomixDocument;
import com.search.supersearch.model.pojo.PlatomixTemplate;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.tika.Tika;
import org.apache.tika.metadata.Metadata;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * @author 喜欢番茄火锅
 * @date 2023/10/9 14:46
 */
@Service
@Slf4j
public class EsService {
    @Resource
    private ElasticsearchClient elasticsearchClient;
    @Resource
    private TikaService tikaService;

    public static String[] getNullPropertyNames(Object source) {
        final BeanWrapper src = new BeanWrapperImpl(source);
        java.beans.PropertyDescriptor[] pds = src.getPropertyDescriptors();

        Set<String> emptyNames = new HashSet<String>();
        for (java.beans.PropertyDescriptor pd : pds) {
            Object srcValue = src.getPropertyValue(pd.getName());
            if (srcValue == null) {
                emptyNames.add(pd.getName());
            }
        }
        String[] result = new String[emptyNames.size()];
        return emptyNames.toArray(result);
    }

    //封装同名称属性复制，但是空属性不复制过去
    public static void copyPropertiesIgnoreNull(Object src, Object target) {
        BeanUtils.copyProperties(src, target, getNullPropertyNames(src));
    }

    public DocsIndex getByFileId(String fileId) throws IOException {
//        List<IamSearchCategoryidIndexname> list = iamSearchCategoryidIndexnameService.list();
//        List<String> collect = list.stream().map(v -> v.getIndexName()).collect(Collectors.toList());

        SearchResponse<DocsIndex> search = elasticsearchClient.search(s -> {
                    SearchRequest.Builder index = s.index("");
                    index.query(q -> q.term(v -> v.field("es_fileId").value(fileId)));
                    return index;
                }
                ,
                DocsIndex.class);
        List<Hit<DocsIndex>> hits = search.hits().hits();
        for (Hit<DocsIndex> hit : hits) {
            return hit.source();
        }
        return null;
    }


    public Integer saveFile(File file, String index, PlatomixDocument platomixDocument) throws Exception {
        String content = null;
        String detect = null;
        Metadata metadata = new Metadata();
        try {
            Tika tika = new Tika();
            tika.setMaxStringLength(-1);
            detect = tika.detect(file);
            content = tikaService.parser(file, metadata);
//        if (detect.contains("image") || detect.contains("pdf")) {
            if (detect.contains("image")) {
//                content = ocrService.ocr(file, detect.split("/")[1]);
            }
            DocsIndex docsIndex = getDocsIndex(index, platomixDocument, content);

            DocsIndex.MetaInfo metaInfo = new DocsIndex.MetaInfo();
            metaInfo.setMetaAuthor(metadata.get("meta:last-author"));
            metaInfo.setMetaModifier(metadata.get("dcterms:modified"));
            metaInfo.setMetaCreated(metadata.get("dcterms:created"));
            metaInfo.setMetaContentType(tika.detect(file));
            metaInfo.setMetaFilesize(file.length() + "");
            metaInfo.setMetaFilename(file.getName());
            metaInfo.setMetaFileUrl(file.getPath());

            Map<String, Object> metaInfoMap = BeanUtil.beanToMap(metaInfo);

            log.info("===== 狗崽的 data =====");
            log.info(JSONUtil.toJsonStr(metaInfoMap));
            docsIndex.setData(metaInfoMap);
            saveFile(index, docsIndex);
        } catch (Exception e) {
            log.info("异常文件={},异常信息={}", file.getName(), e.getMessage());
//            e.printStackTrace();
            return 5;
        }
        return 1;
    }

    private DocsIndex getDocsIndex(String index, PlatomixDocument platomixDocument, String content) throws Exception {
        DocsIndex docsIndex = new DocsIndex();
        docsIndex.setEs_data_content(content);
        docsIndex.setEs_data_simple_content(content);
        if (StringUtils.isNotEmpty(content) && content.length() > 300) {
            docsIndex.setEs_data_simple_content(content.substring(0, 200));
        }

        docsIndex.setEs_data_source(platomixDocument.getCateName());
        docsIndex.setEs_data_type("0");
        docsIndex.setEs_data_title(platomixDocument.getName());
        docsIndex.setEs_data_time(DateUtil.formatDateTime(platomixDocument.getCreateTime()));
        docsIndex.setEs_order_time(platomixDocument.getCreateTime());

        docsIndex.setEs_fileId(IdUtil.fastSimpleUUID());
        docsIndex.setEs_index_creat_date(new Date());
//        docsIndex.setEs_file_path(dwdField.getOssFileFullPath().substring(dwdField.getOssFileFullPath().lastIndexOf("/") + 1));
        docsIndex.setEs_file_path(platomixDocument.getPath());
        docsIndex.setEs_index_name(index);

        docsIndex.setEs_ord_id(platomixDocument.getCateId().toString());
        return docsIndex;
    }

    public void saveFile(String index, DocsIndex docsIndex) throws IOException {
        elasticsearchClient.index(i -> i.index(index).document(docsIndex));
    }


}
